use core::panic;
use std::{fs, io::Write};

use headless_chrome::Browser;
use headless_chrome::types::PrintToPdfOptions;
use nanobyte_opentelemetry::rocket::TracingSpan;
use rocket::{get, fs::NamedFile};
use tracing::{info_span, info};
use crate::chromium::rocket::BrowserHolder;

fn generate_pdf(browser: Browser) {
    let tab = browser.new_tab().unwrap();
    info_span!("open_pdf").in_scope(|| {
       tab.navigate_to("file:///home/6a6996c0-1609-48b6-8ca6-affbef1b4d1d/Devel/Nanobyte/ovlach/ovlach_pdf/template.html").unwrap().wait_until_navigated().unwrap();
    });

    let options = PrintToPdfOptions{
        margin_bottom: Some(0.0),
        margin_left: Some(0.0),
        margin_right: Some(0.0),
        margin_top: Some(0.0),
        print_background: Some(true),
        //paper_width: Some(29.7),
        //paper_height: Some(21.0),
        ..PrintToPdfOptions::default()
    };

    //thread::sleep(Duration::from_secs(10)); 

    let bytes = info_span!("print_pdf").in_scope(|| {
         tab.print_to_pdf(Some(options)).unwrap()
    });

    info_span!("write_temporary_file").in_scope(|| {
        let mut file = fs::OpenOptions::new()
            // .create(true) // To create a new file
            .write(true)
            // either use the ? operator or unwrap since it returns a Result
            .open("/tmp/foo.pdf").unwrap();
        file.write_all(&bytes).unwrap();
    });
}

#[get("/cv/<username>/pdf")]
pub async fn render_pdf_cv(username: &str, browser: BrowserHolder, tracing: TracingSpan) -> Option<NamedFile> { 
    let entered_span = tracing.0.enter();
    let span = info_span!("render_pdf", username = username);
    info!("generate PDF");
    span.in_scope(||{
        generate_pdf(browser.browser);
    });
    info!("done generating");
    drop(entered_span);
    Some(NamedFile::open("/tmp/foo.pdf").await.expect("failed to open foo.pdf"))
}